==> lam_6.1.orig.tar.gz: lam-6.1.orig/share/tstdio/tfopen.c <==
/*
 *	Ohio Trollius
 *	(public domain software)
 *
 *	$Log:	tfopen.c,v $
 * Revision 6.1  96/11/24  00:35:58  nevin
 * Ohio Release
 * 
 * Revision 6.0  96/02/29  14:06:30  gdburns
 * Ohio Release
 * 
 * Revision 5.2  94/08/22  14:17:34  gdburns
 * Ohio Release
 * 
 * Revision 5.1  94/05/18  12:58:49  gdburns
 * Ohio Release
 * 
 * Revision 2.3  94/04/22  12:46:54  gdburns
 * Ohio Release
 * 
 * Revision 2.2.1.1  94/02/16  14:34:34  raja
 * Use the CONST macro.
 * 
 * Revision 2.2  93/05/10  16:38:56  raja
 * pre-release to 2.3
 * 
 *	We are just putting our stamp on this for housekeeping reasons.
 */

/*
 * This file is originally from dLibs 1.2 by Dale Schumacher et al.
 * It was modified for Trollius by MB, 88/12.
 * MB: added call to atkexit() 891012
 * It was modified again for portability by GDB, 91/08.
 * And yet again by JRV for LAM, 93/05.
 */

#include <fcntl.h>

#include <portable.h>
#include <tstdio.h>

#define isiodev(d) ((d)<3)	/* instead of isatty(d) */
#define isstdout(d) ((d)==1)
#define isstderr(d) ((d)==2)

/*
 * external variables
 */
extern int		errno;
extern int		_ioexit;

/*
 * external functions
 */
extern void		tfcloseall();
extern int		iomap_open();

/*
 * INTERNAL FUNCTION.  Attempt to open <filename> in the given
 * <mode> and attach it to the stream <fp>
 */
static TFILE *
_fopen(filename, mode, fp)
	char *filename;
	char *mode;
	TFILE *fp;
{
	int h, i, iomode = 0, f = 0;

	if (! _ioexit) {
		if (atkexit (tfcloseall, (char *) 0) != 0)
			return (NULL);
		_ioexit = 1;
	}

	/*
	 * The CRLF --> \n translation left in fgetc/fputc in case we ever go
	 * back to MS-DOS.  To compensate we default to binary mode here.
	 */
	f |= _TIOBIN;

	while(*mode) {
		switch(*mode++) {
			case 'r':
				f |= _TIOREAD;
				break;
			case 'w':
				f |= _TIOWRT;
				iomode |= (O_CREAT | O_TRUNC);
				break;
			case 'a':
				f |= _TIOWRT;
				iomode |= (O_CREAT | O_APPEND);
				break;
			case '+':
				f |= (_TIOREAD | _TIOWRT);
				break;
			case 'b':
				f |= _TIOBIN;
				break;
			case 't':
				f &= ~_TIOBIN;
				break;
			case 'T':
				f |= _TIOREMOTE;
				break;
			default:
				return(NULL);
		}
	}

	if((i = (f & (_TIOREAD | _TIOWRT))) == 0)
		return(NULL);
	else if(i == _TIOREAD)
		iomode |= O_RDONLY;
	else if(i == _TIOWRT)
		iomode |= O_WRONLY;
	else
		iomode |= O_RDWR;

	h = iomap_open(f, filename, iomode, 0644);

	if(h < 0) {
		errno = h;
		return(NULL);		/* file open/create error */
	}
	if(isiodev(h)) {
		f |= _TIODEV;
		if (isstderr(h))
			f |= _TIONBF;
		else if (isstdout(h))
			f |= _TIOLBF;
		else
			f |= _TIOFBF;
	} else
		f |= _TIOFBF;
	fp->_file = h;			/* file handle */
	fp->_flag = f;			/* file status flags */
	fp->_base = NULL;		/* base of file buffer */
	fp->_ptr = NULL;		/* current buffer pointer */
	fp->_bsiz = 0;			/* buffer size */
	fp->_cnt = 0;			/* # of bytes in buffer */

	return(fp);
}

TFILE *
tfopen(filename, mode)
	CONST char *filename, *mode;
{
	int i;
	TFILE *fp = NULL;

	for(i=0; (!fp && (i < _TNFILE)); ++i)
		if(!(_tiob[i]._flag & (_TIOREAD | _TIOWRT))) /* empty slot? */
			fp = &_tiob[i];
	if(fp)
		return(_fopen(filename, mode, fp));
	return(NULL);
}

TFILE *
tfreopen(filename, mode, fp)
	CONST char *filename, *mode;
	TFILE *fp;
{
	if(tfclose(fp))
		return(NULL);
	return(_fopen(filename, mode, fp));
}

==> lam_6.3.2.orig.tar.gz: lam-6.3.2/share/tstdio/tfopen.c <==
/*
 *
 * Copyright 1998-1999, University of Notre Dame.
 * Authors: Jeffrey M. Squyres, Kinis L. Meyer with M. D. McNally 
 *          and Andrew Lumsdaine
 *
 * This file is part of the Notre Dame LAM implementation of MPI.
 *
 * You should have received a copy of the License Agreement for the
 * Notre Dame LAM implementation of MPI along with the software; see
 * the file LICENSE.  If not, contact Office of Research, University
 * of Notre Dame, Notre Dame, IN 46556.
 *
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 *
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.  
 *
 * Additional copyrights may follow.
 *
 *	Ohio Trollius
 *
 *	$Id: tfopen.c,v 6.5 1999/06/09 17:44:43 jsquyres Exp $
 * 
 *	We are just putting our stamp on this for housekeeping reasons.
 */

/*
 * This file is originally from dLibs 1.2 by Dale Schumacher et al.
 * It was modified for Trollius by MB, 88/12.
 * MB: added call to atkexit() 891012
 * It was modified again for portability by GDB, 91/08.
 * And yet again by JRV for LAM, 93/05.
 */

#include <errno.h>
#include <fcntl.h>

#include <kreq.h>
#include <laminternal.h>
#include <portable.h>
#include <tstdio.h>

#define isiodev(d) ((d)<3)	/* instead of isatty(d) */
#define isstdout(d) ((d)==1)
#define isstderr(d) ((d)==2)

/*
 * external variables
 */
extern int		_ioexit;

/*
 * INTERNAL FUNCTION.  Attempt to open <filename> in the given
 * <mode> and attach it to the stream <fp>
 */
static TFILE *
_fopen(filename, mode, fp)
	char *filename;
	char *mode;
	TFILE *fp;
{
	int h, i, iomode = 0, f = 0;

	if (! _ioexit) {
		if (atkexit (tfcloseall, (char *) 0) != 0)
			return (NULL);
		_ioexit = 1;
	}

	/*
	 * The CRLF --> \n translation left in fgetc/fputc in case we ever go
	 * back to MS-DOS.  To compensate we default to binary mode here.
	 */
	f |= _TIOBIN;

	while(*mode) {
		switch(*mode++) {
			case 'r':
				f |= _TIOREAD;
				break;
			case 'w':
				f |= _TIOWRT;
				iomode |= (O_CREAT | O_TRUNC);
				break;
			case 'a':
				f |= _TIOWRT;
				iomode |= (O_CREAT | O_APPEND);
				break;
			case '+':
				f |= (_TIOREAD | _TIOWRT);
				break;
			case 'b':
				f |= _TIOBIN;
				break;
			case 't':
				f &= ~_TIOBIN;
				break;
			case 'T':
				f |= _TIOREMOTE;
				break;
			default:
				return(NULL);
		}
	}

	if((i = (f & (_TIOREAD | _TIOWRT))) == 0)
		return(NULL);
	else if(i == _TIOREAD)
		iomode |= O_RDONLY;
	else if(i == _TIOWRT)
		iomode |= O_WRONLY;
	else
		iomode |= O_RDWR;

	h = lam_iomap_open(f, filename, iomode, 0644);

	if(h < 0) {
		errno = h;
		return(NULL);		/* file open/create error */
	}
	if(isiodev(h)) {
		f |= _TIODEV;
		if (isstderr(h))
			f |= _TIONBF;
		else if (isstdout(h))
			f |= _TIOLBF;
		else
			f |= _TIOFBF;
	} else
		f |= _TIOFBF;
	fp->_file = h;			/* file handle */
	fp->_flag = f;			/* file status flags */
	fp->_base = NULL;		/* base of file buffer */
	fp->_ptr = NULL;		/* current buffer pointer */
	fp->_bsiz = 0;			/* buffer size */
	fp->_cnt = 0;			/* # of bytes in buffer */

	return(fp);
}

TFILE *
tfopen(filename, mode)
	CONST char *filename, *mode;
{
	int i;
	TFILE *fp = NULL;

	for(i=0; (!fp && (i < _TNFILE)); ++i)
		if(!(_tiob[i]._flag & (_TIOREAD | _TIOWRT))) /* empty slot? */
			fp = &_tiob[i];
	if(fp)
		return(_fopen((char*) filename, (char*) mode, fp));
	return(NULL);
}

TFILE *
tfreopen(filename, mode, fp)
	CONST char *filename, *mode;
	TFILE *fp;
{
	if(tfclose(fp))
		return(NULL);
	return(_fopen((char*) filename, (char*) mode, fp));
}
